#include "../global.h"

Centipede::Centipede(Epp *e, ScreenOutput *disp, AudioOutput *au, Keyboard *kb, Joystick *js, Mouse *mo) :
                                               Machine(e, disp, au) {

    keyboard=kb;
    joystick=js;
    mouse=mo;

    vsync=0;

    for (int i=0; i<0x10000; i++) {

        memory[i]=0xff;
    }
}

void Centipede::readROMs(void) {

    readROM("roms/centiped.307", memory+0x2000, 0x0800);
    readROM("roms/centiped.308", memory+0x2800, 0x0800);
    readROM("roms/centiped.309", memory+0x3000, 0x0800);
    readROM("roms/centiped.310", memory+0x3800, 0x0800);
    readROM("roms/centiped.310", memory+0xf800, 0x0800);

    readROM("roms/centiped.211", charROM+0x0000, 0x0800);
    readROM("roms/centiped.212", charROM+0x0800, 0x0800);

  	dip1=epp->getSetting("centipede", 0);
  	dip1|=epp->getSetting("centipede", 1)<<2;
  	dip1|=epp->getSetting("centipede", 2)<<4;
  	dip1|=epp->getSetting("centipede", 3)<<6;
}

void Centipede::createComponents(void) {

    cpu = new M6502(this);
    vid = new CentipedeVideo(this, display, charROM);
    pokey = new Pokey(this, 1, audio);
}

void Centipede::destroyComponents(void) {

    delete cpu;
    delete vid;
    delete pokey;
}

void Centipede::emulate() {

    readROMs();

    createComponents();
    vid->setupDisplay();
    vid->createBanks();

    cpu->reset();
    cpu->setRunTime(1000000/60);

    for(;;) {

        joystick->update();
        mouse->update();

        vsync=0x40;

        cpu->execute();
        cpu->interrupt(M6502::IRQ);

        pokey->update();
        vid->update();
        audio->update();

        getInput();

        if(epp->checkInterface()) {

            destroyComponents();
            return;
        }
    }
}

void Centipede::reset(void) {

    cpu->reset();
}

uint8 Centipede::readMem(const Component *comp, const uint16 addr) {

    // 6502 memory, incluing mapped input

    uint8 data=0;
	static uint8 pokey_ran=rand();

    // Normal RAM and ROM addresses
    if((addr <= 0x07ff) || (addr>=0x2000 && addr <=0x3fff) || (addr >= 0xf800)) {

    	return memory[addr];
    }
    // Memory mapped input devices

    else if(addr==0x0c00) {

        data=0x20;
    	data |= vsync;
        vsync=0x00;

        return data | htrack;
    }
    else if(addr==0x0c01) {

        if(keyboard->keyPressed(Keyboard::k_3)) data |= 0x80;
        if(joystick->jfire1() || mouse->button1()) data |= 0x04;
        if(keyboard->keyPressed(Keyboard::k_1)) data |= 0x01;
        if(keyboard->keyPressed(Keyboard::k_2)) data |= 0x02;

    	return ~data;
    }
    else if(addr==0x0c02) {

        return vtrack;
    }
    else if(addr==0x0c03) {

        if(joystick->jright()) data |= 0x80;
        if(joystick->jleft()) data |= 0x40;
        if(joystick->jdown()) data |= 0x20;
        if(joystick->jup()) data |= 0x10;

    	return ~data;
    }
    // Dip switches

    else if(addr==0x0800) {

        return dip1;
    }
    else if(addr==0x0801) {

    	return 0x02;
    }
    // Pokey - This is read to get a random number

    else if(addr>=0x1000 && addr<=0x100F) {

		return pokey->readRegister(0, addr);
    }
    return memory[addr];
}

void Centipede::writeMem(const Component *comp, const uint16 addr, const uint8 data) {


    // RAM:
    if (addr<=0x03ff) {

        memory[addr]=data;
    }
    // Special:

    // Background

    else if (addr>=0x0400 && addr<=0x07bf) {

        if(data != memory[addr]) {

            memory[addr]=data;
            vid->writeVideoRAM(addr-0x0400, data);
        }
    }
    // Sprite RAM

    else if (addr>=0x07c0 && addr <=0x7ff) {

    	memory[addr]=data;
    }
    // This should be the pokey

    else if(addr>=0x1000 && addr<=0x100F) {

		pokey->writeRegister(0, addr, data);
    }
    // Bg colour

	else if(addr>=0x1404 && addr<=0x1407) {

        if(data != memory[addr]) {

            memory[addr]=data;

        	uint8 lum=((~data)&0x08)?0xff:0xbb;
            uint8 red=lum*((~data&0x01)>>0);
            uint8 green=lum*((~data&0x02)>>1);
            uint8 blue=lum*((~data&0x04)>>2);

            display->setPalette(addr-0x1404, ((int)red)<<16 | ((int)green)<<8 | blue);
        }
    }
    // Fg colour

	else if(addr>=0x140c && addr<=0x140f) {

        if(data != memory[addr]) {

            memory[addr]=data;

        	uint8 lum=((~data)&0x08)?0xff:0xbb;
            uint8 red=lum*((~data&0x01)>>0);
            uint8 green=lum*((~data&0x02)>>1);
            uint8 blue=lum*((~data&0x04)>>2);

            display->setPalette(addr-0x140c+4, ((int)red)<<16 | ((int)green)<<8 | blue);
        }
    }
}

uint8 Centipede::in(const Component *comp, const uint16 port) {

    // No IO - 6502!
    return 0;
}

void Centipede::out(const Component *comp, const uint16 port, const uint8 data) {

    // No IO - 6502!
}

void Centipede::getInput(void) {

	// The trackball data should only be updated once per frame.
    // No reason to do anything else, though.

    int8 xoff=mouse->getXAxis();
    static int8 xvalue=0;
    htrack=0;

    if (xoff>7) xoff=7;
    if (xoff<-7) xoff=-7;

    xvalue-=xoff;

    if(xvalue>0x10) xvalue-=0x10;
    if(xvalue<0x00) xvalue+=0x10;

	if(xoff > 0) htrack |= 0x80;
    htrack |= xvalue;

    int8 yoff=-mouse->getYAxis();
    static int8 yvalue=0;
	vtrack=0;

    if (yoff>7) yoff=7;
    if (yoff<-7) yoff=-7;

    yvalue-=yoff;

    if(yvalue>0x10) yvalue-=0x10;
    if(yvalue<0x00) yvalue+=0x10;

	if(yoff > 0) vtrack |= 0x80;
    vtrack |= yvalue;
}
